//Kolmogorovův - Smirnovův test pro spojitá rozdělení
//Testuje, zda data mají dané konkrétní spojité rozdělení. 
//Předpokládá, že při výběru onoho konkrétního rozdělení nebyla použita 
//žádná informace z dat (např. střední hodnota nebo rozptyl). 
//Jinak je spočtená p-hodnota nepřesná.
//Test vyžaduje nainstalovaný balíček Distfun.

function [p_hodnota,T,z_alpha]=ks_test_spojity(F,alpha)
    //p_hodnota ... p-hodnota
    //T ... statistika náhodné veličiny
    //z_alpha ... kritická hodnota
    //F ... vektor hodnot distribuční funkce spočtených v hodnotách dat
        //Pokud např. máme data X=[3,6,8] a testujeme shodu s normálním 
        //rozdělením N(5,4), pak hodnoty distribuční funkce získáme příkazem:
        //F=distfun_normcdf(X,5,sqrt(4));
    //alpha ... hladina významnosti
    if argn(2)==1 then
        alpha=0.05;
    end

    [m,n]=size(F);
    N=m*n;
    F=matrix(F,1,N);    
    F1=gsort(F,'c','i');
    F2=(1:N)/N;
    DF1=max(F2-F1);
    DF2=max(F1(2:N)-F2(1:N-1));
    T=max(DF1,DF2);
    z_alpha=distfun_ksinv(alpha,N);
    p_hodnota=1-distfun_kscdf(T,N);
    
endfunction



function [p_hodnota,T,z_alpha]=ks_test_diskretni(F,F_minus_jedna,alpha)
    //p_hodnota ... p-hodnota
    //T ... hodnota testové statistiky
    //z_alpha ... kritická hodnota
    //F ... vektor hodnot distribuční funkce spočtených v hodnotách dat
        //Pokud např. máme data X=[3,6,8] a testujeme shodu s Poissonovým 
        //rozdělením Po(10), pak hodnoty distribuční funkce získáme příkazem:
        //F=distfun_poisscdf(X,10);
    //F_minus jedna ... vektor hodnot distribuční funkce 
    //spočtených v hodnotách předchozích, než jsou skutečná data.
        //Pokud např. máme data X=[3,6,8] a testujeme shodu s Poissonovým
        //rozdělením Po(10), pak hodnoty distribuční funkce získáme takto:
        //
        //XX=X-1;
        //for i=1:100
           //if XX(i)>=0
               //FF(1,i)=distfun_poisscdf(XX(i),10);
           //else
               //FF(1,i)=0;
           //end;
        //end;
        
          //Takto složitě to děláme proto, protože distribuční funkce 
          //pro Poissonovo rozdělení dává pro hodnotu -1 error.
    //alpha ... hladina významnosti

    if argn(2)==2 then
        alpha=0.05;
    end
    [m,n]=size(F);
    N=m*n;

    F1=matrix(F,1,N);
    F2=matrix(F_minus_jedna,1,N);

    Krok=(1+sqrt(5))/4/%pi;
    E=(1:N).*(N+1:2*N)*Krok;
    e=E-fix(E);
        
    F4=F2+(F1-F2).*e;
    
    [p_hodnota,T,z_alpha]=ks_test_spojity(alpha,F4);
        
endfunction



//Kolmogorovův - Smirnovův test pro dva výběrové soubory
//Testuje, zda data z obou výběrů mají stejné rozdělení. 
//Test vyžaduje nainstalovaný balíček Distfun.

function [p_hodnota,T,z_alpha]=ks_test_2(X1,X2,alpha)
    //p_hodnota ... p-hodnota
    //T ... statistika náhodné veličiny
    //z_alpha ... kritická hodnota
    //X1, X2 ... první resp. druhý výběrový soubor
    //alpha ... hladina významnosti
    if argn(2)==2 then
        alpha=0.05;
    end
    
    [m,n]=size(X1);
    N1=m*n;
    X1=matrix(X1,1,N1);    
    X1=gsort(X1,'c','i');
    F1=(1:N1)/N1;
    Z1=zeros(1,N1);
        
    [m,n]=size(X2);
    N2=m*n;
    X2=matrix(X2,1,N2);    
    X2=gsort(X2,'c','i');        
    F2=(1:N2)/N2;
    Z2=zeros(1,N2);

    X=[X1,X2];
    F1=[F1,Z2];
    F2=[Z1,F2];
    
    [X,k]=gsort(X,'c','i');
    F1=F1(k);
    F2=F2(k);
    
    //M=[X;F1;F2];
    
    for i=N1+N2:-1:2
        if X(i)==X(i-1)
            X(i-1)=[];
            F1(i)=max(F1(i),F1(i-1));
            F1(i-1)=[];
            F2(i)=max(F2(i),F2(i-1));
            F2(i-1)=[];
        end;
    end;
    
    [m,n]=size(X);
    
    for i=1:n
        F1(i)=max(F1(1:i));
        F2(i)=max(F2(1:i));
    end;
    
    T=max(abs(F1-F2));
    
    z_alpha=distfun_ksinv(alpha,N);
    p_hodnota=1-distfun_kscdf(T,N);
    
endfunction
